// Address.cpp
//
// Author: Mike McCauley (mikem@airspayce.com)
// Copyright (C) 2013 Mike McCauley
// $Id: Address.cpp,v 1.1 2013/07/13 03:12:23 mikem Exp $

#include "Address.h"

// For memcpy():
#include <string.h>
#include <stdio.h>

Map27Address::Map27Address()
{
    _prefix = MAP27_ADDRESS_DUMMYP;
    _ident  = MAP27_ADDRESS_DUMMYI;
    _adesc  = MAP27_ADDRESS_ADESC_NO_ADDRESS;
    _alen = 0;
}

void     Map27Address::setPrefix(uint8_t prefix)
{
    _prefix = prefix;
}

uint8_t  Map27Address::prefix() const
{
    return _prefix;
}

void     Map27Address::setIdent(uint16_t ident)
{
    _ident = ident;
}

uint16_t Map27Address::ident() const
{
    return _ident;
}

void     Map27Address::setAddress(uint8_t adesc, uint8_t* address, uint8_t alen)
{
    _adesc = adesc;
    _alen = alen;
    if (_alen > MAP27_ADDRESS_MAX_ALEN)
	_alen = MAP27_ADDRESS_MAX_ALEN; // Truncate with no warning
    if (address)
	memcpy(_address, address, _alen);
}

uint8_t  Map27Address::adesc() const
{
    return _adesc;
}

uint8_t  Map27Address::alen() const
{
    return _alen;
}

 const uint8_t* Map27Address::address() const
{
    return _address;
}

uint8_t  Map27Address::decodeBasic(uint8_t* buf, uint16_t bufLen)
{
    if (bufLen >= 3)
    {
	_prefix = buf[0];
	_ident = (buf[1] << 8) | buf[2];
	_adesc = MAP27_ADDRESS_ADESC_NO_ADDRESS;
	_alen = 0;
	return 3;
    }
    else
	return 0;
}

uint8_t  Map27Address::decode(uint8_t* buf, uint16_t bufLen)
{
    if (bufLen >= 4)
    {
	_prefix = buf[0];
	_ident = (buf[1] << 8) | buf[2];
	_adesc = buf[3] >> 4;
	_alen = buf[3] & 0xf;
	if (_alen + 4 <= bufLen)
	{
	    memcpy(_address, buf+4, _alen);
	    return _alen + 4;
	}
    }
    return 0; // Insufficient data
}

uint8_t  Map27Address::encodeBasic(uint8_t* buf, uint16_t bufLen) const
{
    if (bufLen > 3)
    {
	buf[0] = _prefix;
	buf[1] = _ident >> 8;
	buf[2] = _ident & 0xff;
	return 3;
    }
    return 0; // Insufficient buffer space
}

uint8_t  Map27Address::encode(uint8_t* buf, uint16_t bufLen) const
{
    if (bufLen > (4 + _alen))
    {
	buf[0] = _prefix;
	buf[1] = _ident >> 8;
	buf[2] = _ident & 0xff;
	buf[3] = (_adesc << 4) | (_alen & 0xf);
	memcpy(buf+4, _address, _alen);
	
	return 4 + _alen;
    }
    return 0; // Insufficient buffer space
}

uint16_t Map27Address::mpt1343_prefix() const
{
    return (_prefix + 200);
}

uint16_t Map27Address::mpt1343_fin(uint16_t ibi) const
{
    return (ibi / 2) + 2000;	
}

uint16_t Map27Address::mpt1343_un3d(uint16_t ibi) const
{
    return _ident - ibi + 200;
}

uint16_t Map27Address::mpt1343_un2d(uint16_t ibi) const
{
    return _ident - ibi + 20;
}

const char* Map27Address::asString()
{
    static char buf[100];
    snprintf(buf, sizeof(buf), "Prefix %d; Ident %d", _prefix, _ident);
    return buf;
}
