// FCS.h
//
// Author: Mike McCauley (mikem@airspayce.com)
// Copyright (C) 2013 Mike McCauley
// $Id: Address.h,v 1.2 2013/07/14 07:15:46 mikem Exp mikem $

#ifndef MAP27_FCS_H
#define MAP27_FCS_H

#include <stdint.h>

/////////////////////////////////////////////////////////////////////
/// \class Map27FCS FCS.h <Map27/FCS.h>
/// \brief Class for calculating Map27 Frame Check Sequence
///
/// This class provides methods for generating Map27 compliant Frame Check Sequence
/// digits from Map27 message streams. Based on Map27 Appendix A1.
/// This essentially the same as CRC-16-ANSI. 
/// The generator is A001. The result is a 16 bit FCS.
/// It is used internally by Map27PhysicalLayer
/// and you should not normally need to us it in application code.
class Map27FCS
{
public:
    
    /// Constructor
    /// Initialise the FCS for the start of a new calculation
    Map27FCS();

    /// Re-Initialise the FCS for the 
    /// start of a new calculation
    void init();

    /// Add a single octet to the FCS
    /// \param[in] ch The octet to add to the FCS
    void update(uint8_t ch);

    /// Adds a number of octets to the FCS
    /// \param[in] buf Pointer to the buffer of octets to add
    /// \param[in] len Number of octets in buf
    void update(uint8_t* buf, uint16_t len);

    /// Compute and return the FCS
    /// \return The FCS of all the octets passed by update()
    uint16_t final();

    /// Easy to use static class member that will conclate and return the FCS of a number of octets
    /// \param[in] buf Pointer to the buffer of octets to add
    /// \param[in] len Number of octets in buf
    /// \return The FCS of all the octets passed.
    static uint16_t fcs(uint8_t* buf, uint16_t len);

private:
    uint16_t    _fcs;
};


#endif
